// Copyright 2004,2005 Luke Campagnola <lcampagn@mines.edu>, Colorado School of Mines
// 
// This file is part of VRManager.
// 
// VRManager is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// VRManager is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with VRManager; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef VRM_COMM_H
#define VRM_COMM_H

#include <slist.h>

// inter-class communication used by most other classes to inform each other of changes

struct depData {
  class Comm* comm;
  int mesg;
};

class Comm {
  private:
  
      // List of all Comm objects who depend on the state of _this_
    slist<depData> dependents;

  public: 
  
    Comm();
      // Function to be executed when dependencies have changed
    virtual void depChanged( int mesg = 0 );
      // Tell all dependents that _this_ has changed
    void informUpdate( int mesg = 0);
      // add another Comm object to the dependents list
    void addDependent( Comm * comm, int mesg=0 );
      // Clear all dependency information
    void clearDeps();

};

#endif

