// Copyright 2004,2005 Luke Campagnola <lcampagn@mines.edu>, Colorado School of Mines
// 
// This file is part of VRManager.
// 
// VRManager is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// VRManager is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with VRManager; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef VRM_SURFACE_H
#define VRM_SURFACE_H

#include "vectormath.h"
#include "Comm.h"
#include "vrmObject.h"
#include <slist.h>
#include "hashMap.h"
#include "xmlHelpers.h"





class Surface : public Comm, public vrmObject {
  private:
    slist<class Viewport*> viewports;
    
    Vector origin, normal;
    float d;
    bool changed;
  protected:
    void setPlane(Vector o, Vector n);
    void setPlane(Vector a, Vector b, Vector c);
    virtual void update();
    void setMasks();
  
  public: 
    Surface();
    Surface(char* name, VRManager* vrm);
    virtual bool isSimilar(Surface*);
    virtual void addViewport(Viewport* v);
    virtual Vector intersect(Vector a, Vector b);
    virtual void drawMask(Viewport* vp);
    virtual void reconfigure(xmlNode* node) = 0;
};

Surface* createNewSurface(xmlNode* node, class VRManager* vrm);




class Surface_Alias : public Surface {
  private:
    Surface* alias;
    
  public: 
    Surface_Alias();
    Surface_Alias(xmlNode* node, VRManager* vrm);
    bool isSimilar(Surface*);
    void addViewport(class Viewport* v);
    Vector intersect(Vector a, Vector b);
    void drawMask(class Viewport* vp);
    void reconfigure(xmlNode* node);
};




class Surface_2Point : public Surface {
  private:
    class Pointer *o, *n;
    void update();
    
  public: 
    Surface_2Point();
    Surface_2Point(xmlNode *node, VRManager* vrm);
    Surface_2Point(Pointer* o, Pointer* n, VRManager* vrm, char* name=NULL);
    void setPointers(Pointer* o, Pointer* n);
    void depChanged(int msg=0);
    void reconfigure(xmlNode* node);
};

class Surface_3Point : public Surface {
  private:
    class Pointer *a, *b, *c;
    void update();
    
  public: 
    Surface_3Point();
    Surface_3Point(xmlNode *node, VRManager* vrm);
    Surface_3Point(Pointer* a, Pointer* b, Pointer* c, VRManager* vrm, char* name=NULL);
    void setPointers(Pointer* a, Pointer* b, Pointer*c);
    void depChanged(int msg=0);
    void reconfigure(xmlNode* node);
};

  // Mask set needs to be a separate class from Surface to prevent dependency loops
// class MaskSet : public Comm {
//   private:
//     struct maskPt {
//       Vector v;
//       float a;
//     };
//     
//     struct mask {
//       slist<maskPt> pts;
//     };
//     
//     struct maskGroup {
//       map<const char*, mask*, ltstr> maskHash;
//       Viewport* vp;
//       bool valid;
//     };
//     
//     struct pt {
//       float t;
//       int poly;
//       int vtx;
//       int ivtx;
//       float it;  // probably don't need this
//     };
//     
//     typedef quadPair Vector[2][4];
//     
//     
//     map<const char*, maskGroup*, ltstr> maskHash;
//     void invalidateMasks();
//     void updateMask(char* name);    
// 
//       // Functions used in finding quad intersections
//     mask quadIntersect(quadPair quads);
//     int pointWithinQuad(Vector ul, Vector ur, Vector bl, Vector br, Vector pt);
//     int pointWithinQuad(slist<Vector>* a, Vector pt);
//     int pointInOtherQuad(int a, int b);
//     int pointInOtherQuad(pt a);
//     bool segmentIntersect(Vector a1, Vector a2, Vector b1, Vector b2, float* t1, float* t2, Vector* i = NULL);
//     pt nextPt(pt a, pt last);
//     int rot(int a, int mod);
//     Vector ptPos(pt a);
//       
//   public: 
//     MaskSet();
//     void addViewport();
//     void drawMask(char* name);
//     void depChanged(int msg=0);
// }

#endif
