// Copyright 2004,2005 Luke Campagnola <lcampagn@mines.edu>, Colorado School of Mines
// 
// This file is part of VRManager.
// 
// VRManager is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// VRManager is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with VRManager; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef VRM_TRACKER_H
#define VRM_TRACKER_H

#include "vectormath.h"
#include <slist.h>
#include "Comm.h"

  // Holds information about a tracker (such as a single hiball) and its relationship to the virtual coordinate system
class Tracker : public Comm {
  private:
  
    Matrix coordSystem;  // The Matrix that transforms Vectors from the virtual coord system to the tracker coord system
    
      // This data records the state of the trackers
    Vector trackerLocation;
    Vector trackerOrigin;
    Matrix trackerOrientation;
    bool isValid;
    void depChanged(int msg);
    
        
  public:
    
    Tracker();
    void setOrigin(const Vector pos);
    void setLocation(const Vector pos);
    void setOrientationMatrix(const Matrix m);
    Matrix* getTransformMatrix();
    
    void setCoordSystem(Matrix newCS);
};

#endif
