// Copyright 2004,2005 Luke Campagnola <lcampagn@mines.edu>, Colorado School of Mines
// 
// This file is part of VRManager.
// 
// VRManager is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// VRManager is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with VRManager; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef VRM_TRACKERSYSTEM_H
#define VRM_TRACKERSYSTEM_H

#include "vectormath.h"
#include "Tracker.h"
#include "hashMap.h"
#include "xmlHelpers.h"
#include "vrmObject.h"

  // The supported types of tracking systems
enum tsType { 
  TSTYPE_VRPN_TRACKER, 
  TSTYPE_VRPN_BUTTON 
};



  // Holds information about a tracker system (such as a vrpn server)
class TrackerSystem : public vrmObject {
  protected:
    
    Matrix coordSystem;
    tsType type;
//    hash_map<const char*, Tracker*,  hash<const char*>, eqstr> trackerHash;
    map<const char*, Tracker*, ltstr> trackerHash;
    char* trackerName;
    Tracker* parseTracker(xmlNode *node);
    
      // For button event systems  
    void (*buttonCallback)(int,int,void*);
    void* buttonCbUserData;

  public:

    virtual void reconfigure(xmlNode* node, class VRManager* vrm) = 0;
    virtual void cleanup() = 0;
    TrackerSystem();
    ~TrackerSystem();
//     void addTracker(const char* name, Tracker* tracker);
    Tracker* getTracker(const char *name);
    void setButtonCallback(void (*callbackFn)(int, int, void*), void* userData=NULL);
    void setCoordSystem(Matrix newCS);
    friend TrackerSystem* createNewTrackerSystem(xmlNode* node, class VRManager* vrm);

};



  // VRPN Tracking systems
  
#ifdef HAVE_VRPN


#include <vrpn/vrpn_Tracker.h>
#include <vrpn/vrpn_Button.h>
#include <quat/quat.h>


class TS_VrpnTracker : public TrackerSystem {
  private:
    
    
      // VRPN
    vrpn_Tracker_Remote *vrpnTkr;
    bool gotVrpnReport;
    pthread_t vrpnThread;
    int vrpnPollInterval;   // in usec
    map<const char*, Tracker*, ltstr> trackerIdHash;
    bool threadExit;
    
  
  public:

    void cleanup();
    TS_VrpnTracker();
    TS_VrpnTracker(xmlNode* node, class VRManager* vrm);
    void reconfigure(xmlNode* node, class VRManager* vrm);
    void addTracker(Tracker* tracker, const char* name, char* vrpn_id);
    
      // VRPN tracker callbacks
    static void handleVrpnTrackerWrapper(void* ts, vrpn_TRACKERCB b);
    void handleVrpnTracker(const vrpn_TRACKERCB t);
    static void *vrpnTrackerUpdateWrapper(void* ts);
    void vrpnTrackerUpdate();    

};


  // Holds information about a tracker system (such as a vrpn server)
class TS_VrpnButton : public TrackerSystem {
  private:
      // VRPN
    vrpn_Button_Remote *vrpnBtn;
    bool gotVrpnReport;
    pthread_t vrpnThread;
    int vrpnPollInterval;   // in usec
    bool threadExit;
  
  public:

    void cleanup();
    TS_VrpnButton();
    TS_VrpnButton(xmlNode* node, class VRManager* vrm);
    void reconfigure(xmlNode* node, class VRManager* vrm);
      
      // VRPN button callbacks
    static void handleVrpnButtonWrapper(void* ts, vrpn_BUTTONCB b);
    void handleVrpnButton(const vrpn_BUTTONCB t);
    static void *vrpnButtonUpdateWrapper(void* ts);
    void vrpnButtonUpdate();    
};

#endif   // ifdef HAVE_VRPN


#endif
