// Copyright 2004,2005 Luke Campagnola <lcampagn@mines.edu>, Colorado School of Mines
// 
// This file is part of VRManager.
// 
// VRManager is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// VRManager is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with VRManager; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef VRM_VIEWPORT_H
#define VRM_VIEWPORT_H

#include "vectormath.h"
#include "Pointer.h"
#include "Surface.h"  
#include "vrmObject.h"
#include "Comm.h"
#include "xmlHelpers.h"

  
// This class holds information about the location and orientation of a viewport
class Viewport : public Comm, public vrmObject {

  private:
  
    Matrix coordSystem;  //  The Matrix that transforms Vectors from the tracker coord system to the screen coord system
                        //   Z = up, X = left, Y = in
        
    class Pointer* ur_pointer, *ul_pointer, *br_pointer, *bl_pointer;  // Pointers to screen corners
//    Vector left_screen, right_screen, bottom_screen, top_screen;       // These Vectors are the centers of the screen edges in the screen's coordinate system
    float bb_left, bb_right, bb_top, bb_bottom; // parameters of screen's bounding box in screen's coordSystem  
    Vector ur, ul, br, bl;                                   // These Vectors hold the actual coordinates of the screen corners
    Vector ul_u, ur_u, bl_u, br_u;
    Vector ul_w, ur_w, bl_w, br_w;
    bool forceRectangular, useDistortion, allowBlending;
    Vector center, normal;
    int drawArea[4];
    void depChanged(int msg);

    class Surface* surface;
    Matrix distortionMatrix; // undistorts non-rectangular displays
    int polarization;  // left = 0, right = 1
    bool pm_valid, cs_valid;
    void init();
    void update();  // recreate the orientation Matrix and frustum parameters
                  //  (run every time one of the screen corners moves)

  public:
  
    Viewport();
    Viewport(Pointer* uln, Pointer* urn, Pointer* bln, Pointer* brn, int polarization = 0);
    Viewport(xmlNode* node, class VRManager* vrm);
    virtual void reconfigure(xmlNode* node, class VRManager* vrm);
    
    virtual Matrix generateProjectionMatrix(Pointer* eye, float near, float far);  // Generate a GL projection Matrix using eye
    virtual Matrix generateOrthoMatrix(float near, float far);  // Generate an orthographic matrix
    virtual void setCorners(Pointer* uln, Pointer* urn, Pointer* bln, Pointer* brn);
    virtual void correctDistortion(int x, int y, int w, int h);
    virtual void drawMask();
    virtual Pointer* getCorner(int c);
    virtual Pointer* getCorner(char* c);

    virtual void setFastDistortion(bool);
    virtual void setForceRectangular(bool);
    virtual void setPolarization(int p);
    virtual void setSurface(Surface* s);
    virtual Surface* getSurface();
    virtual Vector* getCenter();
    virtual Vector* getNormal();
    virtual int getPolarization();
    virtual int* getDrawArea();
    friend Viewport* createNewViewport(xmlNode* node, class VRManager* vrm, char* name);
};


class Viewport_Alias : public Viewport {
  private: 
    Viewport* alias;
    
  public:
    Viewport_Alias();
    Viewport_Alias(xmlNode* node, class VRManager* vrm);
    void reconfigure(xmlNode* node, class VRManager* vrm);
    
    Matrix generateProjectionMatrix(Pointer* eye, float near, float far);
    Matrix generateOrthoMatrix(float near, float far);
    void setCorners(Pointer* uln, Pointer* urn, Pointer* bln, Pointer* brn);
    void correctDistortion(int x, int y, int w, int h);
    void drawMask();
    Pointer* getCorner(int c);
    Pointer* getCorner(char* c);

    void setFastDistortion(bool);
    void setForceRectangular(bool);
    void setPolarization(int p);
    void setSurface(Surface* s);
    Surface* getSurface();
    Vector* getCenter();
    Vector* getNormal();
    int getPolarization();
    int* getDrawArea();
};

#endif
