// Copyright 2004,2005 Luke Campagnola <lcampagn@mines.edu>, Colorado School of Mines
// 
// This file is part of VRManager.
// 
// VRManager is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// VRManager is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with VRManager; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "debug.h"
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

msgType minDebugLevel = M_INVAL;
char *dbgColorRed, *dbgColorYellow, *dbgColorDef;

void debug_init() {
  dbgColorRed = new char[6];
  dbgColorYellow = new char[6];
  dbgColorDef = new char[21];
  sprintf(dbgColorRed, "%c[31m", 27);
  sprintf(dbgColorYellow, "%c[33m", 27);
  sprintf(dbgColorDef, "%c[22;24;25;27;29;37m", 27);
  
  
  minDebugLevel = M_WARN;  // Default value
  
  char* debugLevel = getenv("VRMANAGER_DEBUG");
  if( debugLevel != NULL ) {
    if( strcmp(debugLevel, "DEBUG") == 0 ) {
      minDebugLevel = M_DEBUG;
    }
    else if( strcmp(debugLevel, "INFO") == 0 ) {
      minDebugLevel = M_INFO;
    }
    else if( strcmp(debugLevel, "WARN") == 0 ) {
      minDebugLevel = M_WARN;
    }
    else if( strcmp(debugLevel, "ERR") == 0 ) {
      minDebugLevel = M_ERR;
    }
  }
}

void msg(msgType t, const char* fmt, ...) {
  if( minDebugLevel == M_INVAL ) {
    debug_init();
  }
  
  if( t < minDebugLevel || fmt == NULL ) {
    return;
  }

  if( t == M_WARN ) {
    fprintf(stderr, "%sWARNING:%s ", dbgColorYellow, dbgColorDef);
  }
  else if( t == M_ERR ) {
    fprintf(stderr, "%sERROR:%s ", dbgColorRed, dbgColorDef);
  }
  
  va_list ap;
  va_start(ap, fmt);
  vfprintf(stderr, fmt, ap);
  va_end(ap);
}


void printMem(void* start, long size) {
  int wpl = 4;
  printf("Memory from 0x%x to 0x%x (+%d)\n", start, (long)start+size, size);
  for( long i=0; i < size; i+= wpl*4 ) {
    printf("0x%x:   ", (long)start+i);
    for( long j=0; j<wpl*4; j++) {
      int v = (*(char*)((long)start+i+j) + 512) % 256;
      v = ((v & 0xf0) >> 4) + ((v & 0x0f) << 4);
      printf("%02x", v);
      if( (j%4) == 3 ) {
        printf(" ");
      }
    }
    printf("  ");
    for( long j=0; j<wpl; j++) {
      int v = *(int*)((long)start+i+(j*4));
      printf("%12u ", v);
      if( (j%4) == 3 ) {
        printf(" ");
      }
    }
    printf("  ");
    for( long j=0; j<wpl*4; j++) {
      char v = *(char*)((long)start+i+j);
//      v = ((v & 0xf0) >> 4) + ((v & 0x0f) << 4);
//      if( v < 0x20 || v > 0x7f ) {
      if( v < 0x20 ) {
        v = '.';
      }
      printf("%c", v);
      if( (j%4) == 3 ) {
        printf(" ");
      }
    }
    printf("\n");
  }
  printf("----\n");
}


int ptrSize(void* p) {
  long l = ((long)p - 8);
  int* i = (int*)l;
  return *i;
}


bool indexValid(void* p, int i, int s) {
  int alloc = ptrSize(p);
  if( (i+1) * s > alloc - 4 ) {
    return false;
  }
  return true;
}
