#include <GL/glut.h>
#include <GL/gl.h>
#include <stdio.h>
#include <vectormath/vectormath.h>
#include <vrmanager/vrmanager.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include "hashMap.h"
#include "xmlHelpers.h"
#include <list.h>
void handleButton(int button, int state, void* ud);

struct dstNodeData {
  xmlNode* node;
  int indent;
  char* text;
  char* name;
  bool tracked;
  Vector p;
};

struct srcNodeData {
  xmlNode* node;
  Vector p;
  char* text;
};

class Globals {
  public:

//  hash_map<const char*, xmlDocPtr,  hash<const char*>, eqstr> xmlDocHash;
  hash_map<const char*, dstNodeData*,  hash<const char*>, eqstr> dstPointerNode;
  // Lists of src/dst nodes
  list<srcNodeData> srcNodeList;
  list<dstNodeData> dstNodeList;
  
  // temporary dst node holder to iterate over when dst is a viewport
  list<dstNodeData*> dstViewportNodeList;
  list<char*> viewports;
  
  VRManager *vrm;
  int screenWidth, screenHeight;
  int numListLines;
  // active cursor frame (left or right)
  int curFrame;
  // ? 
  float srcDiff;
    
  srcNodeData* srcNode;
  dstNodeData* dstNode;
  // Selection location
  list<srcNodeData>::iterator srcSelect;
  list<dstNodeData>::iterator dstSelect;
  
  // Viewport Cursor
  list<dstNodeData*>::iterator dstViewportCur;
  // Cursor location
  list<srcNodeData>::iterator srcCur;
  list<dstNodeData>::iterator dstCur;
  // First node from list *being displayed*
  list<srcNodeData>::iterator srcStart;
  list<dstNodeData>::iterator dstStart;
  
  
  
  
  
  Globals() {
    srcDiff = 1;
    vrm = new VRManager(NULL);
    vrm->setButtonCallback(handleButton);
    
    
    readConfig();
    srcSelect = srcNodeList.begin();
    dstSelect = dstNodeList.begin();
    srcStart = srcNodeList.begin();
    dstStart = dstNodeList.begin();
    srcCur = srcNodeList.begin();
    dstCur = dstNodeList.begin();
    numListLines = 15;
    curFrame = 0;
    srcNode = NULL;
    dstNode = NULL;
  }
  
  void curUp() {
    if( curFrame == 0 ) {
      if( srcCur != srcNodeList.begin() ) {
        srcCur--;
        setSrcStart();
      }
    }
    else if( curFrame == 1 ) {
      if( dstCur != dstNodeList.begin() ) {
        dstCur--;
        setDstStart();
      }
    }
  }
  void curDn() {
    if( curFrame == 0 ) {
      if( srcCur != (--srcNodeList.end()) ) {
        srcCur++;
        setSrcStart();
      }
    }
    else if( curFrame == 1 ) {
      if( dstCur != (--dstNodeList.end()) ) {
        dstCur++;
        setDstStart();
      }
    }
  }
  void curLt() {
    curFrame = 1 - curFrame;
  }
  void curRt() {
    curFrame = 1 - curFrame;
  }
  void enter() {
    if( curFrame == 0 ) {
      srcSelect = srcCur;
    }
    else if( curFrame == 1 ) {
      dstSelect = dstCur;
      if( xmlNameEq(dstSelect->node, "viewport") ) {
        setViewportNodeList(dstSelect->node);
      }
      else {
        dstViewportCur = NULL;
      }
    }  
    setSrcDstNodes();
  }
  
  void dstMove(int axis, float d) {
    if( dstNode == NULL )
      return;
    
    Vector p = strToVector(xmlGetProp(dstNode->node, "vector"));
    p[axis] += d;
    
    char* buf = new char[26];
    sprintf(buf, "%0.4f, %0.4f, %0.4f", p[0], p[1], p[2]);
    xmlSetProp(dstNode->node, "vector", buf);
    saveNode(dstNode->node);
    dstNode->text = xmlNodeString(dstNode->node);
    delete[] buf;
  }
  
    
  void setSrcStart() {
    srcStart = srcCur;
    int i=0; 
    while( i<numListLines/2 && srcStart != srcNodeList.begin() ) {
      srcStart--;
      i++;
    }
  }
  void setDstStart() {
    dstStart = dstCur;
    int i=0; 
    while( i<numListLines/2 && dstStart != dstNodeList.begin() ) {
      dstStart--;
      i++;
    }  
  }
  
  void setViewportNodeList(xmlNode* vp) {
    dstViewportNodeList.clear();
    if( nodeHasChildren(vp) ) {
      char* name = xmlGetProp(vp, "name");
      char* buf = new char[strlen(name)+5];
      sprintf(buf, "%s_ul", name);
      dstViewportNodeList.push_back(dstPointerNode[buf]);
      sprintf(buf, "%s_ur", name);
      dstViewportNodeList.push_back(dstPointerNode[buf]);
      sprintf(buf, "%s_bl", name);
      dstViewportNodeList.push_back(dstPointerNode[buf]);
      sprintf(buf, "%s_br", name);
      dstViewportNodeList.push_back(dstPointerNode[buf]);
printf("added %s to vpnodelist (%x)\n", buf, dstPointerNode[buf]);
    }
    else {
      dstViewportNodeList.push_back(dstPointerNode[xmlGetProp(vp, "ul")]);
      dstViewportNodeList.push_back(dstPointerNode[xmlGetProp(vp, "bl")]);
      dstViewportNodeList.push_back(dstPointerNode[xmlGetProp(vp, "ur")]);
      dstViewportNodeList.push_back(dstPointerNode[xmlGetProp(vp, "br")]);
    }
    dstViewportCur = dstViewportNodeList.begin();
  }
  
  void addSrcNode(xmlNode* node) {
    srcNodeData tmp;
    tmp.node = node;
    tmp.p = vrm->getPointerLocation(xmlGetProp(node, "name"));
    tmp.text = NULL;
    char* buf = new char[strlen(xmlGetProp(node, "name"))+1];
    sprintf(buf, "%s", xmlGetProp(node, "name"));
    tmp.text = buf;
    srcNodeList.push_back(tmp);
  }
  
  void addDstNode(xmlNode* node, int indent, bool tracked, char* prefix = NULL) {
//fprintf(stderr, "add dst node, prefix=%s\n", prefix);
    dstNodeData tmp;
    tmp.tracked = false;
    if( xmlNameEq(node, "pointer") ) {
      if( tracked && ! nodeHasProp(node, "type") ) {
        tmp.tracked = true;
      }
    }
    tmp.node = node;
    tmp.indent = indent;
    tmp.text = xmlNodeString(node);
    if( prefix == NULL ) {
      tmp.name = xmlGetProp(node, "name");
    }
    else {
      tmp.name = new char[strlen(prefix) + strlen(xmlGetProp(node, "name")) +2];
      sprintf(tmp.name, "%s_%s", prefix, xmlGetProp(node, "name"));
    }
//printf("Added dst node %s\n", tmp.name);
    dstNodeList.push_back(tmp);
    if( xmlNameEq(node, "pointer") ) {
      dstPointerNode[tmp.name] = &(*(--dstNodeList.end()));
    }
  }
  
  void readConfig() {
//    char* filename = vrm.getConfigFile();    
    char* filename = "/etc/vrmanagerConfig.xml";
      
      // Check environment variable
    char* envConfigFile = getenv("VRMANAGER_CONFIG_FILE");
    if( envConfigFile != NULL ) {
      filename = envConfigFile;
    }
    parseFile(filename);
  }
  
  void parseFile(char* filename, int indent=0, bool tracked=false, char* prefix = NULL) {
    fprintf(stderr, "vrmconfig: parsing file %s\n", filename);
    xmlDocPtr doc = openFile(filename);
    parseNode(xmlDocGetRootElement(doc), indent, tracked, prefix);
  }
  
  void parseNode(xmlNode* root, int indent, bool tracked, char* prefix = NULL) {
    if( root->type == XML_ELEMENT_NODE ) {
      
//fprintf(stderr, "Adding <%s>\n", xmlNodeName(root));
      addDstNode(root, indent, tracked, prefix);
      if( xmlNameEq(root, "tracker") || nodeHasProp(root, "trackerSystem") ) {
        tracked = true;
      }
      if( xmlNameEq(root, "viewport") || xmlNameEq(root, "surface") ) {
        prefix = xmlGetProp(root, "name");
      }
      
      
      else if( xmlNameEq(root, "pointer") ) {
        if( vrm->getPointer(xmlGetProp(root, "name")) != NULL ) {
//fprintf(stderr, "  .. (source node)\n");
          addSrcNode(root);
        }
        else {
//fprintf(stderr, "  .. null node!\n");
        }
      }
      else if( xmlNameEq(root, "include") ) {
        parseFile(xmlGetProp(root, "src"), indent+1, tracked, prefix);
      }
      for (xmlNode* node = root->children; node; node = node->next) {
        parseNode(node, indent+1, tracked, prefix);
      }
    }
  }
  
    
  xmlDocPtr openFile(char* filename) {
    xmlDocPtr doc;
    doc = xmlReadFile(filename, NULL, 0);
//    xmlDocHash[filename] = doc;
    return doc;
  }
/*
  void saveFile(char* filename) {
    xmlDocPtr doc;
    doc = xmlDocHash[filename];
    xmlSaveFile(filename, doc);
  }
  */
  void saveNode(xmlNode* node) {
    fprintf(stderr, "Saving file \"%s\":\n", node->doc->URL);
    fprintf(stderr, "%s\n\n", xmlTreeString(node->doc->children));
    xmlSaveFile((char*)node->doc->URL, node->doc);
  }
  
  void update() {
    for( list<srcNodeData>::iterator i = srcNodeList.begin(); i != srcNodeList.end(); i++ ) {
      Vector start;
      if( srcNode == &(*i) ) {
        start = i->p;
      }
      
      Vector c = vrm->getPointerLocation(xmlGetProp(i->node, "name"));
      
      float f = 10 * (c-i->p).mag();
      f = f>1    ? 1    : f;
      f = f<.01 ? .01 : f;
      
      //float f = 0.01;
      i->p = (i->p * (1.0-f)) + (c*f);
      
      if( srcNode == &(*i) ) {
        srcDiff = (start-i->p).mag();
      }
      
      
      
/*      char* buf = new char[strlen(xmlGetProp(i->node, "name"))+30];
      sprintf(buf, "%s: %0.4f %0.4f %0.4f", xmlGetProp(i->node, "name"), i->p[0], i->p[1], i->p[2]);
      if( i->text != NULL )
        delete[] i->text;
      i->text = buf;*/
    }
  }
  
  void setSrcDstNodes() {
    srcNode = &(*srcSelect);
    
    if( xmlNameEq(dstSelect->node, "pointer") ) {
      dstNode = &(*dstSelect);
    }
    else if( xmlNameEq(dstSelect->node, "viewport") ) {
      dstNode = *dstViewportCur;
    }
    else {
      dstNode = NULL;
      return;
    }  
  }
  
  void buttonPress() {
    if( srcNode == NULL || dstNode == NULL )
      return;
              
    Vector src = srcNode->p;
          
      // transform src if dst is a tracked pointer
    if( dstNode->tracked ) {
      src = (vrm->getPointerTransform(dstNode->name).inverse() * src) + strToVector(xmlGetProp(dstNode->node, "vector"));
    }
    
    char* buf = new char[26];
    sprintf(buf, "%0.4f, %0.4f, %0.4f", src[0], src[1], src[2]);
    xmlSetProp(dstNode->node, "vector", buf);
    saveNode(dstNode->node);
    dstNode->text = xmlNodeString(dstNode->node);
    delete[] buf;
    
    if( xmlNameEq(dstSelect->node, "viewport") ) {
      dstViewportCur++;
      if( dstViewportCur == dstViewportNodeList.end() ) {
        dstViewportCur = dstViewportNodeList.begin();
      }
    }
    setSrcDstNodes();
  }
  
  
} g;




void drawText(float x, float y, char *string) {
  if( string == NULL )
    return;
  int len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++)
  {
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, string[i]);
  }
}

void drawStatus() {
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);
  glColor4f(0.5, 0.5, 0.5, 1);
  
  Vector src;
  if( g.srcNode != NULL ) {
    src = g.srcNode->p;
          
      // transform src if dst is a tracked pointer
    if( g.dstNode != NULL && g.dstNode->tracked ) {
      src = (g.vrm->getPointerTransform(g.dstNode->name).inverse() * src) + strToVector(xmlGetProp(g.dstNode->node, "vector"));
    }
  }
    
  char* buf = new char[500];
  
  sprintf(buf, "%s [%0.4f %0.4f %0.4f]", (g.srcNode == NULL ? "NULL" : xmlGetProp(g.srcNode->node, "name")), src[0], src[1], src[2]);
  drawText(50, g.screenHeight-30, buf);
  sprintf(buf, "->  %s", (g.dstNode == NULL ? "NULL" : xmlGetProp(g.dstNode->node, "name")));
  drawText(400, g.screenHeight-30, buf);
  delete[] buf;

}

void drawDiff() {
  if( g.srcNode == NULL )
    return;
    
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);  
  glBlendFunc (GL_SRC_ALPHA, GL_DST_ALPHA);
  glEnable (GL_BLEND);
  glEnable (GL_ALPHA_TEST);
  glAlphaFunc (GL_ALWAYS, 0.5);
  glEnable(GL_ALPHA);

  
  float d = log((1000*g.srcDiff+1));
  float l = d * 100;
  glColor4f(d, 1-d, 0, 1);
  glBegin(GL_QUADS);
    glVertex2f(500,   300);
    glVertex2f(500+l, 300);
    glVertex2f(500+l, 400);
    glVertex2f(500,   400);  
  glEnd();
    
}
/*
void drawPointer() {
  if( g.srcNode == NULL )
    return;
  
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);  
  glBlendFunc (GL_SRC_ALPHA, GL_DST_ALPHA);
  glEnable (GL_BLEND);
  glEnable (GL_ALPHA_TEST);
  glAlphaFunc (GL_ALWAYS, 0.5);
  glEnable(GL_ALPHA);

  Vector p = g.vrm->getPointerLocation(xmlGetProp(g.srcNode->node, "name"));
  Viewport* v = g.vrm->getViewport("screen0");
  
  
  Vector s1 = v->getCorner("ul")->getLocation();
  Vector s2 = v->getCorner("bl")->getLocation();
  Vector s3 = v->getCorner("ur")->getLocation();
  Vector s4 = v->getCorner("br")->getLocation();
  Vector n = (s4-s1) * (s3-s2);
  Vector s = (Vector(0, 0, 1) * n).norm() * 0.03;
  
  glBegin(GL_TRIANGLES);
    glColor4f(1, 1, 0, 1);
    glVertex3f(p[0], p[1], p[2]);
    p += s;
    p -= Vector(0, 0, .001);
    glColor4f(1, 1, 0, 0);
    glVertex3f(p[0], p[1], p[2]);
    p -= s * 0.33;
    p -= Vector(0, 0, .01);
    glVertex3f(p[0], p[1], p[2]);
  glEnd();
}*/

void doVertex(Vector p) {
  glVertex3f(p[0], p[1], p[2]);
}

void drawSrcFrame() {
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);  
  glBlendFunc (GL_SRC_ALPHA, GL_DST_ALPHA);
  glEnable (GL_BLEND);
  glEnable (GL_ALPHA_TEST);
  glAlphaFunc (GL_ALWAYS, 0.5);
  glEnable(GL_ALPHA);
  
  float line_height = 16;
  float margin = 10;
  Vector tl(50, g.screenHeight-50);
  Vector br(150, g.screenHeight-50-line_height * g.numListLines);

  
  if( g.curFrame == 0 ) {
    glColor4f(.5, .5, .5, .2);
  }
  else {
    glColor4f(.3, .3, .3, .2);
  }
  glBegin(GL_QUADS);
    glVertex2f(tl[0]-margin, tl[1]);
    glVertex2f(br[0], tl[1]);
    glVertex2f(br[0], br[1]-margin);
    glVertex2f(tl[0]-margin, br[1]-margin);
  glEnd();
  
  int n = 0;
  float cr, cg, cb;
  for( list<srcNodeData>::iterator nData = g.srcStart; nData != g.srcNodeList.end() && n<g.numListLines; nData++ ) {
    cr = cg = cb = 0.4;
    if( nData == g.srcSelect ) {
      cr += 0.2;
      cg += 0.2;
    }
    if( nData == g.srcCur && g.curFrame == 0 ) {
      cr += 0.4;
      cg += 0.4;
      cb += 0.4;
    }
    glColor4f(cr, cg, cb, 1);
      
    drawText(tl[0], tl[1] - ((n+1)*16), nData->text);
    n++;
  }
}

void drawDstFrame() {
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);  
  glBlendFunc (GL_SRC_ALPHA, GL_DST_ALPHA);
  glEnable (GL_BLEND);
  glEnable (GL_ALPHA_TEST);
  glAlphaFunc (GL_ALWAYS, 0.5);
  glEnable(GL_ALPHA);
  
  float line_height = 16;
  float margin = 10;
  Vector tl(200, g.screenHeight-50);
  Vector br(1000, g.screenHeight-50-line_height * g.numListLines);

  if( g.curFrame == 1 ) {
    glColor4f(.5, .5, .5, .2);
  }
  else {
    glColor4f(.3, .3, .3, .2);
  }
  glBegin(GL_QUADS);
    glVertex2f(tl[0]-margin, tl[1]);
    glVertex2f(br[0], tl[1]);
    glVertex2f(br[0], br[1]-margin);
    glVertex2f(tl[0]-margin, br[1]-margin);
  glEnd();

  int n = 0;
  float cr, cg, cb;
  for( list<dstNodeData>::iterator nData = g.dstStart; nData != g.dstNodeList.end() && n<g.numListLines; nData++ ) {
    cr = cg = cb = 0.4;
    if( nData == g.dstSelect ) {
      cr += 0.2;
      cg += 0.2;
    }
    if( g.dstViewportCur != NULL && &(*nData) == &(*(*g.dstViewportCur)) ) {
      cb += 0.4;
    }
    if( nData == g.dstCur && g.curFrame == 1 ) {
      cr += 0.4;
      cg += 0.4;
      cb += 0.4;
    }
    if( xmlNameEq(nData->node, "pointer") ) {
      cb+= 0.2;
    }
    if( xmlNameEq(nData->node, "viewport") ) {
      cg+= 0.2;
    }
    glColor4f(cr, cg, cb, 1);
      
    drawText(tl[0]+(nData->indent * 10), tl[1] - ((n+1)*16), nData->text);
    n++;
  }
}

void drawGrid() {
  
  Viewport* v = g.vrm->getViewport(*(g.viewports.begin()));
  if( v == NULL ) {
    return;
  }
    
  glLineWidth(1);
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);
  Vector s1 = v->getCorner("ul")->getLocation();
  Vector s2 = v->getCorner("bl")->getLocation();
  Vector s3 = v->getCorner("ur")->getLocation();
  Vector s4 = v->getCorner("br")->getLocation();
  
  Vector n = (((s2-s1)*(s3-s1)) + ((s3-s4) * (s2-s4))).norm();
  Vector up = Vector(0, 0, 1);
  Vector rt = up * n;
  
  Vector dx = rt.norm() * ((s3-s1) * 0.07).mag();
  Vector dy = up.norm() * ((s1-s2) * 0.07).mag();
  
  glColor4f(0.5, 0.5, 0.5, 1);
  glBegin(GL_LINES);
    Vector a = s1;
    Vector b = s1;
    int gridsize = 40;
    for( int i=0; i<=gridsize; i++ ) {
      Vector c = a - (dy * gridsize);
      Vector d = b + (dx * gridsize);
      glVertex3f(a[0], a[1], a[2]);
      glVertex3f(c[0], c[1], c[2]);
      glVertex3f(b[0], b[1], b[2]);
      glVertex3f(d[0], d[1], d[2]);
      a += dx;
      b -= dy;
    }
  glEnd();
}

void drawScreen(char* screen) {
  Viewport* v = g.vrm->getViewport(*(g.viewports.begin()));
  
  
  Vector s1 = v->getCorner("ul")->getLocation();
  Vector s2 = v->getCorner("ur")->getLocation();
  Vector s3 = v->getCorner("br")->getLocation();
  Vector s4 = v->getCorner("bl")->getLocation();
  
  
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);
  glColor4f(1, 0, 0, 1);
  glLineWidth(3);
  glBegin(GL_LINE_LOOP);
    doVertex(s1);
    doVertex(s2);
    doVertex(s3);
    doVertex(s4);
  glEnd();
}

void drawBorder() {
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);
  glColor4f(0, 0, 1, 1);
  glLineWidth(40);
  for( int i=0; i<2; i++ ) {
    glBegin(GL_LINE_LOOP);
      glVertex3f(0, 0, 0);
      glVertex3f(1, 0, 0);
      glVertex3f(1, 1, 0);
      glVertex3f(0, 1, 0);
    glEnd();
    glColor4f(0, 1, 0, 1);
    glLineWidth(1);
  }
}


void drawGui() {
  glViewport(0, 0, g.screenWidth, g.screenHeight);
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
    glLoadIdentity();
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
      glLoadIdentity();
      glOrtho(0, g.screenWidth, 0, g.screenHeight, -1000, 1000);
      drawSrcFrame();
      drawDstFrame();
      drawStatus();
      drawDiff();
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
  glPopMatrix();
}



void display() {
  glClearColor(0, 0, 0, 0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  for( list<char*>::iterator i = g.viewports.begin(); i != g.viewports.end(); i++ ) {
    glMatrixMode(GL_PROJECTION);
      
    int* drawArea = g.vrm->getViewportDrawArea(*i);
    if( drawArea[0] != -1 ) {
      printf("Setting draw area to %d %d %d %d\n", drawArea[0], drawArea[1], drawArea[2], drawArea[3]);
      glViewport(drawArea[0], drawArea[1], drawArea[2], drawArea[3]);
    }
    else {
      printf("not setting draw area.\n");
    }
    glLoadIdentity();
    glOrtho(0, 1, 0, 1, -1, 1);
    drawBorder();
    
    glLoadMatrixf(g.vrm->getGlOrthoMatrix(*i, .01, 1000));
    
    drawGrid();
    drawScreen(*i);
//    drawPointer();  
//     if( s == 0 ) {
//       g.vrm->adjustBuffer("screen0", "", 0, 0, 1280, 1024);  
//     }
//     else {
//       g.vrm->adjustBuffer("screen1", "", 1280, 0, 1280, 1024);  
//     }    
  }
  drawGui();

  glFinish();
  glutSwapBuffers(); 
}

void idle(void) {
  g.update();
  glutPostRedisplay();
}

void handleButton(int button, int state, void* ud) {
  if( state == 0 ) {
    g.buttonPress();
  }
  //printf("Button %d %s\n", button, (state==0 ? "released" : "pressed"));
}

void keyboard(unsigned char key, int xPos, int yPos) {
  bool shift = glutGetModifiers() & 2 > 0;
  float d = shift ? .001 : .01;
  
  if( key == 27 ) {  // Esc
    exit(0);
  }
  else if( key == 13 ) {  // enter
    g.enter();
  }
  else if( key == 97 || key == 65) { // a
    g.dstMove(0, d);
  }
  else if( key == 122 || key == 90 ) { // z
    g.dstMove(0, -d);
  }
  else if( key == 115 || key == 83 ) { // s
    g.dstMove(1, d);
  }
  else if( key == 120 || key == 88 ) { // x
    g.dstMove(1, -d);
  }
  else if( key == 100 || key == 68 ) { // d
    g.dstMove(2, d);
  }
  else if( key == 99 || key == 67 ) { // c
    g.dstMove(2, -d);
  }
  else {
    printf("Pressed key %d\n", key);
  }
}


void skeyboard(int key, int xPos, int yPos) {
  if( key == 101 ) {  // up
    g.curUp();
  }
  else if( key == 103 ) {  // down
    g.curDn();
  }
  else if( key == 102 ) {  // rt
    g.curRt();
  }
  else if( key == 100 ) {  // lt
    g.curLt();
  }
  else if( key == 104 ) {  // pgup
    for( int i=0; i<g.numListLines-1; i++ )
      g.curUp();
  }
  else if( key == 105 ) {  // pgdn
    for( int i=0; i<g.numListLines-1; i++ )
      g.curDn();
  }
  else {
    printf("Pressed (s)key %d\n", key);
  }
}

void motion(int x, int y) {

}

void mouse(int button, int state, int x, int y) {
  printf("Mouse button %d %s at %d %d\n", button, (state==0 ? "released" : "pressed"), x, y);
}

void reshape(int w, int h) {
  glViewport(0, 0, w, h);
  g.screenWidth = w;
  g.screenHeight = h;
}

void menu(int item) {
}

int main(int argc, char** argv) {
  
  for( int i=1; i<argc; i++ ) {
    g.viewports.push_front(argv[i]);
  }
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
  int mainWindow = glutCreateWindow("Window!");
  glutFullScreen();
  glutDisplayFunc(display);
  glutReshapeFunc(reshape);
  glutKeyboardFunc(keyboard);
  glutSpecialFunc(skeyboard);
  glutMouseFunc(mouse);
  glutMotionFunc(motion);
  glutIdleFunc(idle);
  
    /*
  glutCreateMenu(menu);
  glutAddMenuEntry("Menu entry 1", 1);
  glutAddMenuEntry("", -1);
  glutAddMenuEntry("Menu entry 2", 2);
  glutAddMenuEntry("Exit", 0);
  glutAttachMenu(GLUT_RIGHT_BUTTON);
    */
    
  glutMainLoop();
  
  return 0;
}
